/* ====================================================================
 * Copyright (c) 1995-1999 The Apache Group.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Apache Group
 *    for use in the Apache HTTP server project (http://www.apache.org/)."
 *
 * 4. The names "Apache Server" and "Apache Group" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Apache Group
 *    for use in the Apache HTTP server project (http://www.apache.org/)."
 *
 * THIS SOFTWARE IS PROVIDED BY THE APACHE GROUP ``AS IS'' AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE APACHE GROUP OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Group and was originally based
 * on public domain software written at the National Center for
 * Supercomputing Applications, University of Illinois, Urbana-Champaign.
 * For more information on the Apache Group and the Apache HTTP server
 * project, please see <http://www.apache.org/>.
 *
 */

/* 
 * Info Module.  Display configuration information for the server and
 * all included modules.
 *
 * <Location /online>
 * SetHandler radwho
 * AuthType   Basic
 * AuthName   Radius
 * Require    valid-user
 * </Location>
 *
 * GET /online - Returns who is currently online
 * GET /online/clear?RadAcctId  - deletes the accounting record with the given Id
 * GET /online/data?name=?????  - gets radgroup data and logins for the day
 * GET /online/today            - lists everybody who has logged in today.
 * GET /online/sort?by=????     - sorts the table according to the selected critiria
 * GET /online/showme?id        - shows the entire 'id' record in the database.
 * GET /online/todays?days=n    - Show all logins for the past n days
 *
 * Robert 'Admiral' Coeyman <admiral@corner.net> 2001
 *
 * In 2002 
 *  Added the authentication routines
 *  Cleaned up the look of the page.
 *  Set the refresh in the header.
 *  Created the clearing function for 'stuck' logins.
 *  Made "/clear" case insensitive.
 *  Added the data directive.
 *  Added code to hypothesize if the radius server is running or now.	
 *  Wrote a routine to show the entire login record for a current login.	
 *  Changed the clear instruction to create a logout with an
 *   AcctTerminateCause of 'Dead Login' and a session length of 5 seconds.
 *  [ Otherwise, our synchronization daemon would restore the dead login. ]     
 *
 * May 2002 by Isam Ishaq <isam@planet.edu>  *  Added /sort directive.
 *  Added CallingStationID "Calling From" to the /data table.
 *  Clicking on the user name shows the logins by that user.
 *
 *
 *
 * Built from the mod_info file.
 *
 */

#include "httpd.h"
#include "http_config.h"
#include "http_core.h"
#include "http_log.h"
#include "http_main.h"
#include "http_protocol.h"
#include "util_script.h"
#include "http_conf_globals.h"
#include <stdio.h>
#include <time.h>
#include "radwho.h"
#include <mysql/mysql.h>
#include <gdbm.h>          /* database header.        */
#include <string.h>        /* for strncmp,strstr      */
#include <unistd.h>

typedef struct sql_socket {
	MYSQL *sock;
	MYSQL conn;
	MYSQL_RES *result;
	MYSQL_RES *result2;
	int id;
	int in_use;
	struct timeval tv;
} SQLSOCK;

#define NULL ((void *)0)


module icradius_who_module;


static int display_online(request_rec *r)
{
    SQLSOCK *socket;
    MYSQL_ROW row,brow;	
    char buf[256], string[128], *p,ports[49], sortby[50];
    server_rec *serv = r->server;
    int pct, iii,ii,xxx,comma = 0;
    struct tm *btime,*today;
    time_t now, then, online;
    time_t limit;		
    int res,days,extra;
    FILE *pidfile;
    FILE *Run;		
	

    r->content_type = "text/html";
    ap_send_http_header(r);
    if (r->header_only) {
        return 0;
    }


    ap_hard_timeout("send radius online", r);

// Take out the refresh if this is a data request.
//  You need time to read the data...

if ( strcasecmp(r->path_info,"/data") ) {
    ap_rputs(DOCTYPE_HTML_3_2
	     "<html><head><title>Radius Information</title>\n",r);
    ap_rprintf(r,"<meta http-equiv=\"Refresh\" content=\"300\">\n");
    ap_rputs("</head>\n<body BGCOLOR=\"#33CCff\"><h3 align=center>Icradius Login Information</h1>\n",
r);
	} else {
    ap_rputs(DOCTYPE_HTML_3_2
	     "<html><head><title>Radius Information</title>\n</head>\n", r);
    ap_rputs("<body BGCOLOR=\"#33CCff\"><h3 align=center>Icradius Login Information</h1>\n",
r);
}

socket=malloc(sizeof(SQLSOCK));    /* create the socket           */

if ( socket == NULL ) {
	ap_rputs("SERIOUS problem. Socket got no memory.\n",r);
	ap_rputs("</BODY></HTML>\n",r);
	return 0;
}

mysql_init(&(socket->conn));       /* Activate the socket         */
socket->sock = mysql_real_connect(&(socket->conn),DBHOST,USERNAME,PASSWORD,DATABASE,0,NULL,0);
if (socket->sock == NULL ) {
	ap_rputs("<B>Failed to connect to the radius server</B>",r);
	ap_rprintf(r,"<BR>Error:%s<BR></BODY></HTML>\n",
	mysql_error(socket->sock));
	mysql_free_result(socket->result);
	mysql_close(socket->sock);
	free(socket);
	return 0;
}

/* If we got a command, execute it first. */
if ( r->path_info != NULL ) {
	limit = (time_t)0;

    /* added by Isam start */

         sprintf((char *)&sortby,"NASPortId"); /* Set a default to sort by */

       if( !strcasecmp(r->path_info,"/sort") ) {
                       // With the GET method, the data is included in the args field.
                       //   r->args.  ( have to strip away the by= )
        p = strstr((const char *)r->args,"=");

        if (( p != NULL) && ( socket->sock != NULL) ) {  // error if null
           p++;  /* next character is the sort critiria....  */
             sprintf((char *)&sortby,"%s",p);
             ap_rprintf(r,"Sorted by %s\n",sortby);
          }
             ap_rprintf(r,"Sorted by %s\n",sortby);

         } /* end of the Sorting instruction */

    /* added by Isam end */

	if( !strcasecmp(r->path_info,"/clear") ) {
/*		sprintf((char *)&buf,"DELETE FROM radacct WHERE RadAcctId = %s",r->args); */
	/* new code */
		sprintf((char *)&buf,"UPDATE radacct SET AcctStopTime=NOW(),AcctSessionTime='5',AcctTerminateCause='Dead Login' WHERE RadAcctId = %s",r->args); 
	/* Trap Errors first */
		if (socket->sock == NULL ) {
			ap_rputs("<B>Failed to QUERY the radius server</B>",r);
			ap_rprintf(r,"<BR>Error:%s<BR></BODY></HTML>\n",
			mysql_error(socket->sock));
			mysql_free_result(socket->result);
			mysql_close(socket->sock);
			free(socket);
			return 0;
		} /* end of the error Trap */
		mysql_query(socket->sock,(char *)&buf);
		syslog(LOG_INFO,"Dead login %s removed",r->args);
		ap_rputs("<I>Dead login removed successfully</I><BR>",r);
	} /* end of the clearing instruction */

	if( !strcasecmp(r->path_info,"/data") ) {
			// With the GET method, the data is included in the args field.
/* find out if the days section was sent */
	p = strstr((const char *)r->args,"s=");
	/* ALWAYS send the days LAST */
	if ( p == NULL ) {
			days=1;
		} else {
			p+=2;  /* skip the two characters */
			/* what if the user sends an empty string? */
			if ( *p != (char) 0 ) {
				 days=atoi(p);
			} else {
				 days=1;
			}
		}
	 p = strstr((const char *)r->args,"&");
	 if ( p != NULL) *p = (char) 0;   /* cut the string */		
			//   r->args.  ( have to strip away the name= )
	 p = strstr((const char *)r->args,"e=");
	  if ( p == NULL ) ap_rputs("No name sent<BR>\n",r);

	  if (( p != NULL) && ( socket->sock != NULL) ) {  // error if null
	    p+=2;  /* next character is the name....  */
// Check for a user of this name in our records
	    sprintf((char *)&buf,"SELECT GroupName FROM usergroup WHERE UserName = '%s'",p);
	    mysql_query(socket->sock,(char *)&buf);
            socket->result=mysql_store_result(socket->sock);
	    comma = mysql_num_rows(socket->result);
	if ( comma == 0 ) {
		// Unable to find the user in the group list.
	    sprintf((char *)&buf,"SELECT NASPortId FROM radacct WHERE UserName = '%s'",p);
	    mysql_query(socket->sock,(char *)&buf);
            socket->result=mysql_store_result(socket->sock);
	    comma = mysql_num_rows(socket->result);
        } // end of the second try
if ( comma == 0  ) {
	    ap_rprintf(r,"No such user as %s.<BR><BR>",p);
	    sprintf((char *)&buf,"SELECT DISTINCT UserName FROM usergroup WHERE UserName LIKE '%s%%'",p);
	    mysql_query(socket->sock,(char *)&buf);
            socket->result=mysql_store_result(socket->sock);
	    comma = mysql_num_rows(socket->result);
	    ap_rprintf(r,"%d alternatives found",comma);	
		if ( comma > 0 ) {
		ap_rprintf(r,"<FORM METHOD=GET ACTION=\"%s/data\">\n",MODROOT);
		ap_rputs("<SELECT NAME=\"name\" size=1>\n",r);
			for ( iii=0;iii<comma;iii++) {
                 	        row = mysql_fetch_row(socket->result);
				if ( row[0] != NULL ) ap_rprintf(r,"<OPTION>%s\n",row[0]);
                        }   /* end of the counting loop */
		ap_rputs("</SELECT>\n",r);
		ap_rputs("<INPUT TYPE=\"submit\" VALUE=\"find\">\n",r);
		ap_rputs("</FORM>\n",r); 
		} /* end of the form */
		ap_rputs("</BODY></HTML>\n",r); 
	    mysql_free_result(socket->result);
	    mysql_close(socket->sock);
	    free(socket);
	    return 0;		
} /* No Data trap */
/* Get the user's group */
/* table conversion by Isam */
            mysql_free_result(socket->result);
	    sprintf((char *)&buf,"SELECT GroupName FROM usergroup WHERE UserName = '%s'",p);
	    mysql_query(socket->sock,(char *)&buf);
            socket->result=mysql_store_result(socket->sock);
            row = mysql_fetch_row(socket->result);
	if ( row[0] == NULL ) {
	    sprintf((char *)&buf,"SELECT GroupName FROM usergroup WHERE UserName = 'DEFAULT'");
	    mysql_query(socket->sock,(char *)&buf);
            socket->result=mysql_store_result(socket->sock);
            row = mysql_fetch_row(socket->result);
	}  /* check for a default group */
		if ( row[0] != NULL ) {
                     ap_rputs("<TABLE ALIGN=CENTER BORDER=1 BGCOLOR=\"#ffffff\">\n",r);
	             ap_rprintf(r,"<TR bgcolor=\"#f6debc\"><TH COLSPAN=2>%s is in group %s</th></tr>\n",p,row[0]);
// This is the best place to put the group items.
                    ap_rputs("<TR><TH ALIGN=CENTER COLSPAN=2>Check Items</th></tr>\n",r);
		    sprintf((char *)&buf,"SELECT * FROM radgroupcheck WHERE GroupName = '%s'",row[0]);
		    mysql_query(socket->sock,(char *)&buf);
            	    socket->result2=mysql_store_result(socket->sock);
// Got those now.  Print them.
          		comma = mysql_num_rows(socket->result2);
			for ( iii=0;iii<comma;iii++) {
                 	    brow = mysql_fetch_row(socket->result2);
				if( !strcasecmp(brow[2],"Daily-Time-Limit") ) {
	if ( brow[3] == NULL ) {
                ap_rputs("<TR><TD COLSPAN=2>Time limit with no value?</td></tr>\n",r);
	} else {
		then = (time_t)atol(brow[3]);
		limit = then;
		today = gmtime((time_t *)&then);
                ap_rprintf(r,"<TR><TD>Daily-Time-Limit</td><TD> %dH %dM %dS</td></tr>\n",today->tm_hour,
			today->tm_min,today->tm_sec);
	}
					} else {
                                    ap_rprintf(r,"<TR><TD>%s</td><TD> %s</td></tr>\n",brow[2],brow[3]);

				} // done converting the time limit
                        } // done looping for reply elements.
                        mysql_free_result(socket->result2);
// Get the reply items now.
                       ap_rputs("<TR><TH ALIGN=CENTER COLSPAN=2>Reply Items</th></tr>\n",r);
		    sprintf((char *)&buf,"SELECT * FROM radgroupreply WHERE GroupName = '%s'",row[0]);
		    mysql_query(socket->sock,(char *)&buf);
            	    socket->result2=mysql_store_result(socket->sock);
// Got those now.  Print them.
          		comma = mysql_num_rows(socket->result2);
			for ( iii=0;iii<comma;iii++) {
                 	    brow = mysql_fetch_row(socket->result2);
                             ap_rprintf(r,"<TR><TD>%s</td><TD> %s </td></tr>\n",brow[2],brow[3]);
                        } // done looping for reply elements.
                        ap_rprintf(r,"</TABLE><BR>\n");
                        mysql_free_result(socket->result2);
		} else {
			ap_rprintf(r,"%s is in NO group<BR>\n",p);
		} // end of the group name stuff.	
          }  /* end of the null pointer trap */
          mysql_free_result(socket->result);
        sprintf((char *)&buf,"SELECT NASPortId,AcctStartTime,AcctStopTime,AcctSessionTime,AcctTerminateCause,CallingStationID,RadAcctID FROM radacct WHERE UserName = '%s' AND (TO_DAYS(NOW()) - TO_DAYS(AcctStartTime)) <= %d",p,days);
	mysql_query(socket->sock,(char *)&buf);
	socket->result=mysql_store_result(socket->sock);
	comma = mysql_num_rows(socket->result);
	if ( comma == 0 ) {
		ap_rprintf(r,"%s has not been online in the past %d days.<BR>\n",p,days);	
	} else {
// Create table.
		ap_rputs("<TABLE WIDTH=100\% BORDER=1 BGCOLOR=\"#ffffff\">\n",r);
// Table header
	ap_rputs("<TR>\n",r);
	ap_rputs("<TH>Port</TH>\n",r);
	ap_rputs("<TH>Login Time</TH>\n",r);
	ap_rputs("<TH>LogOut Time</TH>\n",r);
	ap_rputs("<TH>Online</TH>\n",r);
	ap_rputs("<TH>Logout Cause</TH>\n",r);
        ap_rputs("<TH>Calling From</TH>\n",r);
	ap_rputs("</TR>\n",r);
// Loop to fill in the table.
	for ( iii=0;iii<comma;iii++) {
                row = mysql_fetch_row(socket->result);
		ap_rputs("<TR>\n",r);
		ap_rprintf(r,"<TD ALIGN=CENTER>%s</TD>\n",row[0]);
		ap_rprintf(r,"<TD ALIGN=CENTER>%s</TD>\n",row[1]);
		ap_rprintf(r,"<TD ALIGN=CENTER>%s</TD>\n",row[2]);
// Break this down
	if ( row[3] == NULL ) {
		ap_rputs("<TD ALIGN=CENTER>Still Online</TD>\n",r);
	} else {
		then = (time_t)atol(row[3]);
		if ( then == 0 ) {
			ap_rputs("<TD ALIGN=CENTER>Still Online</TD>\n",r);
		    } else {
			today = gmtime((time_t *)&then);
			ap_rprintf(r,"<TD ALIGN=CENTER>%dH %dM %dS</TD>\n",today->tm_hour,
				today->tm_min,today->tm_sec);
		} // inner trap for 0 time used
	}
		ap_rprintf(r,"<TD ALIGN=CENTER>&nbsp <a href=\"%s/showme?%s\">?</a> %s</TD>\n",MODROOT,row[6],row[4]);
                ap_rprintf(r,"<TD ALIGN=CENTER>&nbsp %s</TD>\n",row[5]);
		ap_rputs("</TR>\n",r);
	} // end of the usage listing.
// Table footer.
	mysql_free_result(socket->result);
	ap_rputs("<TR>\n",r);
	ap_rputs("<TH>Port</TH>\n",r);
	ap_rputs("<TH>Login Time</TH>\n",r);
	ap_rputs("<TH>LogOut Time</TH>\n",r);
	ap_rputs("<TH>Online</TH>\n",r);
	ap_rputs("<TH>Logout Cause</TH>\n",r);
        ap_rputs("<TH>Calling From</TH>\n",r);
	ap_rputs("</TR>\n",r);
// Now, get the total time logged.
sprintf((char *)&buf,"SELECT SUM(AcctSessionTime) FROM radacct WHERE UserName = '%s' AND TO_DAYS(AcctStartTime) = TO_DAYS(NOW())",p);
	mysql_query(socket->sock,(char *)&buf);
	socket->result2=mysql_store_result(socket->sock);
	brow=mysql_fetch_row(socket->result2);
	if ( brow[0] == NULL ) {
		ap_rputs("<TR>\n",r);
                ap_rputs("<TH COLSPAN=3 ALIGN=RIGHT>Total Used Today</TH>\n",r);
                ap_rputs("<TH ALIGN=CENTER COLSPAN=3> None </TH>\n",r);
		ap_rputs("</TR>\n",r);
	} else {
		then = (time_t)atol(brow[0]);
		if ( (limit > then) && (limit != 0) ) { 
		ap_rputs("<TR bgcolor=\"#f6debc\">\n",r);
		} else {
		ap_rputs("<TR bgcolor=\"#ff9999\">\n",r);
		}
                ap_rputs("<TH COLSPAN=3 ALIGN=RIGHT>Total Used Today</TH>\n",r);
                today = gmtime((time_t *)&then);
                ap_rprintf(r,"<TD ALIGN=CENTER COLSPAN=3>%dH %dM %dS</TD>\n",today->tm_hour,
                        today->tm_min,today->tm_sec);
		ap_rputs("</TR>\n",r);
/* now list the lime limit for the day */
		if ( limit > 0 ) {
		ap_rputs("<TR bgcolor=\"#f6debc\">\n",r);
		today = gmtime((time_t *)&limit);
		ap_rprintf(r,"<TH COLSPAN=3>Daily-Time-Limit</TH>\n<TD COLSPAN=3 ALIGN=CENTER>%dH %dM %dS</TD>\n",today->tm_hour,
			today->tm_min,today->tm_sec);
		}
		ap_rputs("</TR>\n",r);
	} /* add up the total time used */
/* At this point I want to give a breakdown for the past 30 days. */
		mysql_free_result(socket->result2);
		ap_rputs("<TR><th colspan=6>Login history for the past 30 days</TH></TR>\n",r);
		for ( iii=1;iii<31;iii++) {
			if ( iii == 1 ) { 
		ap_rputs("<TR><th colspan=3>Yesterday</TH>\n",r);
			} else {
		ap_rprintf(r,"<tr><TH COLSPAN=3>%d days ago</TH>\n",iii);
		} /* neatness for the display of days vs. day */
sprintf((char *)&buf,"SELECT SUM(AcctSessionTime) FROM radacct WHERE UserName = '%s' AND TO_DAYS(AcctStartTime) = (TO_DAYS(NOW())-%d)",p,iii);
	/* Run the query */
	mysql_query(socket->sock,(char *)&buf);
	socket->result2=mysql_store_result(socket->sock);
	brow=mysql_fetch_row(socket->result2);
	if ( brow != NULL ) {
		if (brow[0] != NULL) {
		then = (time_t)atol(brow[0]);
                today = gmtime((time_t *)&then);
                ap_rprintf(r,"<TD ALIGN=CENTER COLSPAN=3>%dH %dM %dS</TD>\n",today->tm_hour,
                        today->tm_min,today->tm_sec);
		ap_rputs("</TR>\n",r);
			} else { /* cap for NULL TRAP */
			ap_rputs("<td align=center colspan=3>-----</td></tr>\n",r);
		} /* No data display */
	} else { /* main NULL trap */
			ap_rputs("<td align=center colspan=3>-----</td></tr>\n",r);
               }  
		mysql_free_result(socket->result2);
		} /* done counting to 30 */


		ap_rputs("</Table>\n<BR>",r);
	} // end of the user was online table.
      } /* end of the data printout. instruction */

	if( !strcasecmp(r->path_info,"/today") ) {
		/* start by creating my table */
	ap_rputs("<BR><TABLE WIDTH=100% BORDER=1 BGCOLOR=\"ffffff\">",r);
	ap_rputs("<TR><TH>UserName:</TH><TH>0<BR>0</TH><TH>0<BR>1</TH><TH>0<BR>2</TH>",r);
	ap_rputs("<TH>0<BR>3</TH><TH>0<BR>4</TH><TH>0<BR>5</TH><TH>0<BR>6</TH>",r);
	ap_rputs("<TH>0<BR>7</TH><TH>0<BR>8</TH><TH>0<BR>9</TH><TH>1<BR>0</TH>",r);
	ap_rputs("<TH>1<BR>1</TH><TH>1<BR>2</TH><TH>1<BR>3</TH><TH>1<BR>4</TH>",r);
	ap_rputs("<TH>1<BR>5</TH><TH>1<BR>6</TH><TH>1<BR>7</TH><TH>1<BR>8</TH>",r);
	ap_rputs("<TH>1<BR>9</TH><TH>2<BR>0</TH><TH>2<BR>1</TH><TH>2<BR>2</TH>",r);
	ap_rputs("<TH>2<BR>3</TH><TH>2<BR>4</TH><TH>2<BR>5</TH><TH>2<BR>6</TH>",r);
	ap_rputs("<TH>2<BR>7</TH><TH>2<BR>7</TH><TH>2<BR>9</TH><TH>3<BR>0</TH>",r);
	ap_rputs("<TH>3<BR>1</TH><TH>3<BR>2</TH><TH>3<BR>3</TH><TH>3<BR>4</TH>",r);
	ap_rputs("<TH>3<BR>5</TH><TH>3<BR>6</TH><TH>3<BR>7</TH><TH>3<BR>8</TH>",r);
	ap_rputs("<TH>3<BR>9</TH><TH>4<BR>0</TH><TH>4<BR>1</TH><TH>4<BR>2</TH>",r);
	ap_rputs("<TH>4<BR>3</TH><TH>4<BR>4</TH><TH>4<BR>5</TH><TH>4<BR>6</TH>",r);
	ap_rputs("<TH>4<BR>7</TH><TH>4<BR>8</TH></TR>",r);
/* run the query */
	sprintf((char *)&buf,"SELECT DISTINCT UserName FROM radacct WHERE TO_DAYS(AcctStartTime) = TO_DAYS(NOW()) ORDER BY UserName");
	mysql_query(socket->sock,(char *)&buf);

	socket->result=mysql_store_result(socket->sock);

	if ( socket->result == NULL ) {
		ap_rputs("<TR><TD COLSPAN=50 ALIGN=CENTER>No data to summerize</TD></TR>",r);
	} else {
		comma = mysql_num_rows(socket->result); /* need result count */
		if ( comma != 0 ) { /* the inner nobody online trap */
		for ( iii=0;iii<comma;iii++) { /* per user loop */
			/* zero out the string */
			memcpy((void *)&ports,"...................................................",50);
			row = mysql_fetch_row(socket->result); /* get the username */
			if ( row[0] != NULL ) {
			sprintf((char *)&buf,"SELECT DISTINCT NASPortId FROM radacct WHERE TO_DAYS(AcctStartTime) = TO_DAYS(NOW()) AND UserName='%s'",row[0]);
			mysql_query(socket->sock,(char *)&buf);
			socket->result2=mysql_store_result(socket->sock);
			pct = mysql_num_rows(socket->result2); /* need result count */
			 for ( ii=0;ii<pct;ii++) { /* per port loop */
				brow = mysql_fetch_row(socket->result2); /* get the portid */
				if ( brow[0] != NULL) xxx = atoi(brow[0]);
				if ( xxx < 49 ) ports[xxx] = 'X';
			     } /* end of the per port loop */
			} /* a just in case null trap */
			mysql_free_result(socket->result2);
			ap_rprintf(r,"<TR><TD><A href=\"%s/data?name=%s\">%s</a></TD>\n",MODROOT,row[0],row[0]);
			for ( ii=0;ii<49;ii++) {
				ap_rprintf(r,"<TD>%c</TD>",ports[ii]);
				} /* inner port loop */
			ap_rputs("</TR>",r);
                } /* end of the per user loop */
	   } else {   /* middle of the nobody online trap */
		ap_rputs("<TR><TD COLSPAN=50 ALIGN=CENTER>No logins to summerize</TD></TR>",r);
	   } /* end of the nobody online trap */
	} /* end of the no users to list trap */
	ap_rputs("<TR><TH>UserName:</TH><TH COLSPAN=49>Ports that this user has used today</TH></TR>",r);
	ap_rputs("</TABLE><BR>",r);
/* Done with the table */
	mysql_free_result(socket->result);  /* done with this userlist */
     } /* end of the today instruction */

/* Added by Admiral Coeyman */
	if( !strcasecmp(r->path_info,"/showme") ) {
		sprintf((char *)&buf,"SELECT * FROM radacct WHERE RadAcctId = %s",r->args);
	/* Trap Errors first */
		if (socket->sock == NULL ) {
			ap_rputs("<B>Failed to QUERY the radius server</B>",r);
			ap_rprintf(r,"<BR>Error:%s<BR></BODY></HTML>\n",
			mysql_error(socket->sock));
			mysql_free_result(socket->result);
			mysql_close(socket->sock);
			free(socket);
			return 0;
		} /* end of the error Trap */
		mysql_query(socket->sock,(char *)&buf);
		syslog(LOG_INFO,"Exposing login %s ...",r->args);
		socket->result=mysql_store_result(socket->sock);

/* Not ALL of this data is available for users still online, however,
  	this command doesn't have to only be run for them... */

		if ( socket->result == NULL ) {
			ap_rputs("<B>ERROR: Unable to find that login online.</B>\n",r);
			mysql_free_result(socket->result);
		} else {  /* got data--print it out */
			row = mysql_fetch_row(socket->result);
			if ( row == NULL ) {
				syslog(LOG_ERR,"ERROR:Exposing login %s [%s]...",r->args,buf);
				return 0;
			}
			ap_rputs("<table width=57% bgcolor=\"white\" border=2>\n",r);
			ap_rputs("<tr><th>Accounting ID</th>\n",r);
			if ( row[0] != NULL ) {
				ap_rprintf(r,"<td>&nbsp %s </td></tr>\n",row[0]);
				} else {
				ap_rputs("<td>&nbsp N/A </td></tr>\n",r);
			}
			ap_rputs("<tr><th>Session ID</th>\n",r);
			if ( row[1] != NULL ) {
				ap_rprintf(r,"<td>&nbsp %s </td></tr>\n",row[1]);
				} else {
				ap_rputs("<td>&nbsp N/A </td></tr>\n",r);
			}
			ap_rputs("<tr><th>User Name</th>\n",r);
			if ( row[2] != NULL ) {
				ap_rprintf(r,"<td>&nbsp %s </td></tr>\n",row[2]);
				} else {
				ap_rputs("<td>&nbsp N/A </td></tr>\n",r);
			}
			ap_rputs("<tr><th>Realm</th>\n",r);
			if ( row[3] != NULL ) {
				ap_rprintf(r,"<td>&nbsp %s </td></tr>\n",row[3]);
				} else {
				ap_rputs("<td>&nbsp N/A </td></tr>\n",r);
			}
			ap_rputs("<tr><th>NAS IP Address</th>\n",r);
			if ( row[4] != NULL ) {
				ap_rprintf(r,"<td>&nbsp %s </td></tr>\n",row[4]);
				} else {
				ap_rputs("<td>&nbsp N/A </td></tr>\n",r);
			}
			ap_rputs("<tr><th>Port</th>\n",r);
			if ( row[5] != NULL ) {
				ap_rprintf(r,"<td>&nbsp %s </td></tr>\n",row[5]);
				} else {
				ap_rputs("<td>&nbsp N/A </td></tr>\n",r);
			}
			ap_rputs("<tr><th>Port Type</th>\n",r);
			if ( row[6] != NULL ) {
				ap_rprintf(r,"<td>&nbsp %s </td></tr>\n",row[6]);
				} else {
				ap_rputs("<td>&nbsp N/A </td></tr>\n",r);
			}
			ap_rputs("<tr><th>Login Time</th>\n",r);
			if ( row[7] != NULL ) {
				ap_rprintf(r,"<td>&nbsp %s </td></tr>\n",row[7]);
				} else {
				ap_rputs("<td>&nbsp N/A </td></tr>\n",r);
			}
			ap_rputs("<tr><th>Logout Time</th>\n",r);
			if ( row[8] != NULL ) {
				ap_rprintf(r,"<td>&nbsp %s </td></tr>\n",row[8]);
				} else {
				ap_rputs("<td>&nbsp N/A </td></tr>\n",r);
			}
			ap_rputs("<tr><th>Session Time</th>\n",r);
			if ( row[9] != NULL ) {
				ap_rprintf(r,"<td>&nbsp %s </td></tr>\n",row[9]);
				} else {
				ap_rputs("<td>&nbsp N/A </td></tr>\n",r);
			}
			ap_rputs("<tr><th>Authenticated By</th>\n",r);
			if ( row[10] != NULL ) {
				ap_rprintf(r,"<td>&nbsp %s </td></tr>\n",row[10]);
				} else {
				ap_rputs("<td>&nbsp N/A </td></tr>\n",r);
			}
			ap_rputs("<tr><th>Connection Info</th>\n",r);
			if ( row[11] != NULL ) {
				ap_rprintf(r,"<td>&nbsp %s </td></tr>\n",row[11]);
				} else {
				ap_rputs("<td>&nbsp N/A </td></tr>\n",r);
			}
			ap_rputs("<tr><th>Input Bytes</th>\n",r);
			if ( row[12] != NULL ) {
				ap_rprintf(r,"<td>&nbsp %s </td></tr>\n",row[12]);
				} else {
				ap_rputs("<td>&nbsp N/A </td></tr>\n",r);
			}
			ap_rputs("<tr><th>Output Bytes</th>\n",r);
			if ( row[13] != NULL ) {
				ap_rprintf(r,"<td>&nbsp %s </td></tr>\n",row[13]);
				} else {
				ap_rputs("<td>&nbsp N/A </td></tr>\n",r);
			}
			ap_rputs("<tr><th>Called Station ID</th>\n",r);
			if ( row[14] != NULL ) {
				ap_rprintf(r,"<td>&nbsp %s </td></tr>\n",row[14]);
				} else {
				ap_rputs("<td>&nbsp N/A </td></tr>\n",r);
			}
			ap_rputs("<tr><th>Calling Station ID</th>\n",r);
			if ( row[15] != NULL ) {
				ap_rprintf(r,"<td>&nbsp %s </td></tr>\n",row[15]);
				} else {
				ap_rputs("<td>&nbsp N/A </td></tr>\n",r);
			}
			ap_rputs("<tr><th>Termination Cause</th>\n",r);
			if ( row[16] != NULL ) {
				ap_rprintf(r,"<td>&nbsp %s </td></tr>\n",row[16]);
				} else {
				ap_rputs("<td>&nbsp N/A </td></tr>\n",r);
			}
			ap_rputs("<tr><th>Service Type</th>\n",r);
			if ( row[17] != NULL ) {
				ap_rprintf(r,"<td>&nbsp %s </td></tr>\n",row[17]);
				} else {
				ap_rputs("<td>&nbsp N/A </td></tr>\n",r);
			}
			ap_rputs("<tr><th>Protocol</th>\n",r);
			if ( row[18] != NULL ) {
				ap_rprintf(r,"<td>&nbsp %s </td></tr>\n",row[18]);
				} else {
				ap_rputs("<td>&nbsp N/A </td></tr>\n",r);
			}
			ap_rputs("<tr><th>IP Address</th>\n",r);
			if ( row[19] != NULL ) {
				ap_rprintf(r,"<td>&nbsp %s </td></tr>\n",row[19]);
				} else {
				ap_rputs("<td>&nbsp N/A </td></tr>\n",r);
			}
			ap_rputs("<tr><th>Start Delay</th>\n",r);
			if ( row[20] != NULL ) {
				ap_rprintf(r,"<td>&nbsp %s </td></tr>\n",row[20]);
				} else {
				ap_rputs("<td>&nbsp N/A </td></tr>\n",r);
			}
			ap_rputs("<tr><th>Stop Delay</th>\n",r);
			if ( row[21] != NULL ) {
				ap_rprintf(r,"<td>&nbsp %s </td></tr>\n",row[21]);
				} else {
				ap_rputs("<td>&nbsp N/A </td></tr>\n",r);
			}

/* now get averaged data for this user. */
		sprintf((char *)&buf,"SELECT AVG(AcctSessionTime),MIN(AcctSessionTime),MAX(AcctSessionTime),SUM(AcctSessionTime),STD(AcctSessionTime) FROM radacct WHERE UserName = '%s' AND AcctSessionTime > '0'",row[2]);
		mysql_query(socket->sock,(char *)&buf);
		syslog(LOG_INFO,"Exposing login %s ...",r->args);
		socket->result2=mysql_store_result(socket->sock);

		if ( socket->result2 != NULL ) {	
				brow=mysql_fetch_row(socket->result2);
			} else { 
				brow = NULL;
		}


		if ( brow != NULL ) {
			ap_rputs("<tr><th>Average Login</th>\n",r);
			if ( brow[0] != NULL ) {
				ap_rprintf(r,"<td>&nbsp %s Seconds </td></tr>\n",brow[0]);
				} else {
				ap_rputs("<td>&nbsp N/A </td></tr>\n",r);
			}
			ap_rputs("<tr><th>Minimum Login</th>\n",r);
			if ( brow[1] != NULL ) {
				ap_rprintf(r,"<td>&nbsp %s seconds </td></tr>\n",brow[1]);
				} else {
				ap_rputs("<td>&nbsp N/A </td></tr>\n",r);
			}
			ap_rputs("<tr><th>Maximum Login</th>\n",r);
			if ( brow[2] != NULL ) {
				ap_rprintf(r,"<td>&nbsp %s seconds </td></tr>\n",brow[2]);
				} else {
				ap_rputs("<td>&nbsp N/A </td></tr>\n",r);
			}
			ap_rputs("<tr><th>Total [known] Login Time</th>\n",r);
			if ( brow[3] != NULL ) {
				ap_rprintf(r,"<td>&nbsp %s seconds </td></tr>\n",brow[3]);
				} else {
				ap_rputs("<td>&nbsp N/A </td></tr>\n",r);
			}
			ap_rputs("<tr><th>Standard Deviation</th>\n",r);
			if ( brow[4] != NULL ) {
				ap_rprintf(r,"<td>&nbsp %s seconds</td></tr>\n",brow[4]);
				} else {
				ap_rputs("<td>&nbsp N/A </td></tr>\n",r);
			}
		      } /* end of the NULL brow trap */
			ap_rputs("</table><br><br><br>\n\n",r);
			mysql_free_result(socket->result);
			mysql_free_result(socket->result2);
		} /* end of the printout. */

        } /* end of the showme request. */
	if( !strcasecmp(r->path_info,"/todays") ) {
		/* find out for how many days */
		p = strstr(r->args,"s=");
		p+=2;  /* skip the two characters */
		if ( p == NULL ) {
			days=1;
		} else {
			/* what if the user sends an empty string? */
			if ( *p != (char) 0 ) {
				 days=atoi(p);
			} else {
				 days=1;
			}
		}
		/* start by creating my table */
	ap_rputs("<BR><TABLE WIDTH=100% BORDER=1 BGCOLOR=\"ffffff\">",r);
	ap_rputs("<TR><TH>UserName:</TH><TH>0<BR>0</TH><TH>0<BR>1</TH><TH>0<BR>2</TH>",r);
	ap_rputs("<TH>0<BR>3</TH><TH>0<BR>4</TH><TH>0<BR>5</TH><TH>0<BR>6</TH>",r);
	ap_rputs("<TH>0<BR>7</TH><TH>0<BR>8</TH><TH>0<BR>9</TH><TH>1<BR>0</TH>",r);
	ap_rputs("<TH>1<BR>1</TH><TH>1<BR>2</TH><TH>1<BR>3</TH><TH>1<BR>4</TH>",r);
	ap_rputs("<TH>1<BR>5</TH><TH>1<BR>6</TH><TH>1<BR>7</TH><TH>1<BR>8</TH>",r);
	ap_rputs("<TH>1<BR>9</TH><TH>2<BR>0</TH><TH>2<BR>1</TH><TH>2<BR>2</TH>",r);
	ap_rputs("<TH>2<BR>3</TH><TH>2<BR>4</TH><TH>2<BR>5</TH><TH>2<BR>6</TH>",r);
	ap_rputs("<TH>2<BR>7</TH><TH>2<BR>7</TH><TH>2<BR>9</TH><TH>3<BR>0</TH>",r);
	ap_rputs("<TH>3<BR>1</TH><TH>3<BR>2</TH><TH>3<BR>3</TH><TH>3<BR>4</TH>",r);
	ap_rputs("<TH>3<BR>5</TH><TH>3<BR>6</TH><TH>3<BR>7</TH><TH>3<BR>8</TH>",r);
	ap_rputs("<TH>3<BR>9</TH><TH>4<BR>0</TH><TH>4<BR>1</TH><TH>4<BR>2</TH>",r);
	ap_rputs("<TH>4<BR>3</TH><TH>4<BR>4</TH><TH>4<BR>5</TH><TH>4<BR>6</TH>",r);
	ap_rputs("<TH>4<BR>7</TH><TH>4<BR>8</TH></TR>",r);
/* run the query */
	sprintf((char *)&buf,"SELECT DISTINCT UserName FROM radacct WHERE (TO_DAYS(NOW()) - TO_DAYS(AcctStartTime)) <= '%d' ORDER BY UserName",days);
	mysql_query(socket->sock,(char *)&buf);

	socket->result=mysql_store_result(socket->sock);

	if ( socket->result == NULL ) {
		ap_rputs("<TR><TD COLSPAN=50 ALIGN=CENTER>No data to summerize</TD></TR>",r);
	} else {
		comma = mysql_num_rows(socket->result); /* need result count */
		if ( comma != 0 ) { /* the inner nobody online trap */
		for ( iii=0;iii<comma;iii++) { /* per user loop */
			/* zero out the string */
			memcpy((void *)&ports,"...................................................",50);
			row = mysql_fetch_row(socket->result); /* get the username */
			if ( row[0] != NULL ) {
			sprintf((char *)&buf,"SELECT DISTINCT NASPortId FROM radacct WHERE (TO_DAYS(NOW()) - TO_DAYS(AcctStartTime)) <= '%d'  AND UserName='%s'",days,row[0]);
			mysql_query(socket->sock,(char *)&buf);
			socket->result2=mysql_store_result(socket->sock);
			pct = mysql_num_rows(socket->result2); /* need result count */
			 for ( ii=0;ii<pct;ii++) { /* per port loop */
				brow = mysql_fetch_row(socket->result2); /* get the portid */
				if ( brow[0] != NULL) xxx = atoi(brow[0]);
				if ( xxx < 49 ) ports[xxx] = 'X';
			     } /* end of the per port loop */
			} /* a just in case null trap */
			mysql_free_result(socket->result2);
			ap_rprintf(r,"<TR><TD><A href=\"%s/data?name=%s&days=%d\">%s</a></TD>\n",MODROOT,row[0],days,row[0]);
			for ( ii=0;ii<49;ii++) {
				ap_rprintf(r,"<TD>%c</TD>",ports[ii]);
				} /* inner port loop */
			ap_rputs("</TR>",r);
                } /* end of the per user loop */
	   } else {   /* middle of the nobody online trap */
		ap_rputs("<TR><TD COLSPAN=50 ALIGN=CENTER>No logins to summerize</TD></TR>",r);
	   } /* end of the nobody online trap */
	} /* end of the no users to list trap */
	ap_rprintf(r,"<TR><TH>UserName:</TH><TH COLSPAN=49>Ports that this user has used in the past %d days.</TH></TR>",days);
	ap_rputs("</TABLE><BR>",r);
/* Done with the table */
	mysql_free_result(socket->result);  /* done with this userlist */
     } /* end of the todays instruction */

}  /* end of the commands section */


 /* next line modified by Isam   */
 sprintf((char *)&buf,"SELECT RadAcctId,UserName,NASPortId,AcctStartTime,FramedIPAddress,UNIX_TIMESTAMP(AcctStartTime),ConnectInfo FROM radacct WHERE AcctSessionTime = '0' ORDER BY %s", sortby);


if (socket->sock == NULL ) {
	ap_rputs("<B>Failed to QUERY the radius server</B>",r);
	ap_rprintf(r,"<BR>Error:%s<BR></BODY></HTML>\n",
	mysql_error(socket->sock));
	mysql_free_result(socket->result);
	mysql_close(socket->sock);
	free(socket);
	return 0;
}
mysql_query(socket->sock,(char *)&buf);

socket->result=mysql_store_result(socket->sock);
if ( socket->result == NULL ) {
	ap_rputs("<B>ERROR: Unable to find users online.</B>\n",r);
	ap_rputs("</BODY></HTML>\n",r);
	mysql_free_result(socket->result);
	mysql_close(socket->sock);
	free(socket);
	return 0;
}
comma = mysql_num_rows(socket->result);

if ( comma == 0 ) {
	ap_rputs("<B>No users found online at this time</B>\n",r);
/* Insert the query block anyway. */
	ap_rputs("<TABLE WIDTH=100\% BORDER=1 BGCOLOR=\"#ffffff\">\n",r);
/* Start with the server time. */
now = time(NULL);
/* give the current time  */
sprintf((char *)&string,"<TR><TH COLSPAN=7>Server Time: %s</TH></TR>\n",ctime((time_t *)&now));
ap_rprintf(r,"%s",string);
/* now for the functions */
	ap_rputs("<TR><TH COLSPAN=3 bgcolor=\"#f6debc\">\n",r);
	ap_rprintf(r,"<FORM METHOD=GET ACTION=\"%s/data\">\n",MODROOT);
	ap_rputs("<INPUT NAME=\"name\" size=15>\n",r);
	ap_rputs("<INPUT TYPE=\"submit\" VALUE=\"find\">\n",r);
	ap_rputs("</FORM></TH>\n",r); 
	ap_rprintf(r,"<TH COLSPAN=1 bgcolor=\"#f6debc\"><FORM METHOD=GET ACTION=\"%s/today\">\n",MODROOT);
	ap_rputs("<INPUT TYPE=\"submit\" VALUE=\"Today\">\n",r);
	ap_rputs("</FORM></TH>\n<TH COLSPAN=3 bgcolor=\"#f6debc\">\n",r);
	ap_rprintf(r,"<FORM METHOD=GET ACTION=\"%s/todays\">\n",MODROOT);
	ap_rputs("<INPUT NAME=\"days\" size=2 value=\"1\">\n",r);
	ap_rputs("<INPUT TYPE=\"submit\" VALUE=\"days\">\n",r);
	ap_rputs("</FORM></TD></TR>\n",r); 
	ap_rputs("</TABLE><BR><BR><BR>\n", r);  /* close the table */
/* Done with the query block */
	ap_rputs("</BODY></HTML>\n",r);
	mysql_free_result(socket->result);
	mysql_close(socket->sock);
	free(socket);
	return 0;
}

ap_rputs("<TABLE WIDTH=100\% BORDER=1 BGCOLOR=\"#ffffff\">\n",r);

/* tell how many users are online.   */

if ( comma > 1 ) {
        ap_rprintf(r,"<TR><TH COLSPAN=7 bgcolor=\"#f6debc\">%d Users are online (sorted by %s)</TH></TR>",comma,sortby);
    } else {
	ap_rprintf(r,"<TR><TH COLSPAN=7 bgcolor=\"#f6debc\">1 User is online</TH></TR>");
}

now = time(NULL);

/* give the current time  */

sprintf((char *)&string,"<TR><TH COLSPAN=7>Server Time: %s</TH></TR>\n",ctime((time_t *)&now));
ap_rprintf(r,"%s",string);


  
/* Start with a header */

	ap_rputs("<TR>\n",r);
         /* modified by Isam start */
       ap_rprintf(r,"<TH><a href=\"%s/sort?by=UserName\">UserName</a></TH>\n",MODROOT);
       ap_rprintf(r,"<TH><a href=\"%s/sort?by=NASPortId\">Port</a></TH>\n",MODROOT);
       ap_rprintf(r,"<TH><a href=\"%s/sort?by=AcctStartTime\">Login Time</a></TH>\n",MODROOT);
       ap_rprintf(r,"<TH><a href=\"%s/sort?by=FramedIPAddress\">User IP</a></TH>\n",MODROOT);
         ap_rputs("<TH>Online</TH>\n",r);
         /* modified by Isam end */
	ap_rputs("<TH>Today</TH>\n",r);
        ap_rprintf(r,"<TH><a href=\"%s/sort?by=ConnectInfo\">Info</a></TH>\n",MODROOT);
	ap_rputs("</TR>\n",r);

/* Then print all the data from a loop */

for ( iii=0;iii<comma;iii++) {
	row = mysql_fetch_row(socket->result);
/* Quick convert to get the login time. */
        then = (time_t)atol(row[5]);
	online = now - then;
/* Convert the time online for printing.  */
	btime = gmtime((time_t *)&online);
sprintf((char *)&buf,"SELECT SUM(AcctSessionTime) FROM radacct WHERE UserName = '%s' AND TO_DAYS(AcctStartTime) = TO_DAYS(NOW())",row[1]);
	mysql_query(socket->sock,(char *)&buf);
	socket->result2=mysql_store_result(socket->sock);
	brow=mysql_fetch_row(socket->result2);
	if ( brow[0] == NULL ) {
		then = online;
	} else {
		then = (time_t)atol(brow[0]);
		then += online;
	} /* add the current time used to the total. */


if ( then > USECAP ) {  /* in excess of ten hours. -1 second [default] 35999 */
		ap_rputs("<TR bgcolor=\"#ff9999\">\n",r);
	} else {
		ap_rputs("<TR>\n",r);
}  /* end of the ten hour cap */

        ap_rprintf(r,"<TD ALIGN=CENTER><A HREF=\"%s/data?name=%s\">%s</A></TD>\n",MODROOT,row[1],row[1]);
	ap_rprintf(r,"<TD ALIGN=CENTER>%s</TD>\n",row[2]);
	ap_rprintf(r,"<TD ALIGN=CENTER>%s</TD>\n",row[3]);
	ap_rprintf(r,"<TD ALIGN=CENTER>%s</TD>\n",row[4]);
	ap_rprintf(r,"<TD ALIGN=CENTER>%dH %dM %dS</TD>\n",btime->tm_hour,
		btime->tm_min,btime->tm_sec);
/* Since the space used to store this structure is overwritten
    each time gmtime is called, I cannot call this before writing
    out the earlier data.  Otherwise, this data gets printed in
    both places.
*/
	today = gmtime((time_t *)&then);
	ap_rprintf(r,"<TD ALIGN=CENTER>%dH %dM %dS</TD>\n",today->tm_hour,
		today->tm_min,today->tm_sec);
	ap_rprintf(r,"<TD><a href=\"%s/clear?%s\">X</a>&nbsp <a href=\"%s/showme?%s\">?</a> %s</TD>\n",MODROOT,row[0],MODROOT,row[0],row[6]);
	ap_rputs("</TR>\n",r);
	mysql_free_result(socket->result2);
}

/* Now finish up the table */
	ap_rputs("<TR>\n",r);                  /* repeat the header */
         /* modified by Isam start */
       ap_rprintf(r,"<TH><a href=\"%s/sort?by=UserName\">UserName</a></TH>\n",MODROOT);
       ap_rprintf(r,"<TH><a href=\"%s/sort?by=NASPortId\">Port</a></TH>\n",MODROOT);
       ap_rprintf(r,"<TH><a href=\"%s/sort?by=AcctStartTime\">Login Time</a></TH>\n",MODROOT);
       ap_rprintf(r,"<TH><a href=\"%s/sort?by=FramedIPAddress\">User IP</a></TH>\n",MODROOT);
       ap_rputs("<TH>Online</TH>\n",r);
         /* modified by Isam end */
	ap_rputs("<TH>Today</TH>\n",r);
        ap_rprintf(r,"<TH><a href=\"%s/sort?by=ConnectInfo\">Info</a></TH>\n",MODROOT);
	ap_rputs("</TR>\n",r);
/* Before we close the table, we create the form to get more data
    on a single user.... */
	ap_rputs("<TR><TH COLSPAN=3 bgcolor=\"#f6debc\">\n",r);
	ap_rprintf(r,"<FORM METHOD=GET ACTION=\"%s/data\">\n",MODROOT);
	ap_rputs("<INPUT NAME=\"name\" size=15>\n",r);
	ap_rputs("<INPUT TYPE=\"submit\" VALUE=\"find\">\n",r);
	ap_rputs("</FORM></TH>\n",r); 
	ap_rprintf(r,"<TH COLSPAN=1 bgcolor=\"#f6debc\"><FORM METHOD=GET ACTION=\"%s/today\">\n",MODROOT);
	ap_rputs("<INPUT TYPE=\"submit\" VALUE=\"Today\">\n",r);
	ap_rputs("</FORM></TH>\n<TH COLSPAN=3 bgcolor=\"#f6debc\">\n",r);
	ap_rprintf(r,"<FORM METHOD=GET ACTION=\"%s/todays\">\n",MODROOT);
	ap_rputs("<INPUT NAME=\"days\" size=2 value=\"1\">\n",r);
	ap_rputs("<INPUT TYPE=\"submit\" VALUE=\"days\">\n",r);
	ap_rputs("</FORM></TD></TR>\n",r); 

	ap_rputs("</TABLE><BR><BR><BR>\n", r);  /* close the table */
/* lets give some instructions */
	ap_rputs("<u>Usage:</u><br>\n", r);  
	ap_rputs("Click the table headings to sort the listings.<br>\n", r);  
	ap_rputs("The 'X' in the info column deletes a dead login.<br>\n", r);  
	ap_rputs("The '?' in the info column shows the entire login record for a login.<br>\n", r);  
        ap_rputs("Click the Username to get todays logins for the user.<br>\n", r);

	ap_rputs("<br>\n", r);  /* End of the instructions */

/* Close out my socket  */
	mysql_free_result(socket->result);
	mysql_close(socket->sock);
	free(socket);
	ap_rprintf(r,"<BR>URL:%s \n",r->uri);
	ap_rprintf(r,"<BR>PATH:%s \n",r->path_info);
	ap_rprintf(r,"<BR>ARGS:%s <br>\n",r->args);

/* At this point, I'm working on a way to report if the server is
    still running or not. --Admiral Coeyman */

pidfile = fopen("/var/run/radiusd.pid","r"); /* only need to read it. */
if ( pidfile == NULL ) {
	ap_rputs("<B>Radius pidfile not found.</b><br>\n",r);    	
} else {
	comma = fread((void *)&string,1,13,pidfile);
	comma--;  /* back over the space. */
	string[comma]=(char)0;
	sprintf((char *)&buf,"/proc/%s/stat",string);
/*	ap_rprintf(r,"radius pid:%s [%s]<br>\n",string,buf);  */
	Run=fopen((char *)&buf,"r");
	if ( Run == NULL ) {
		ap_rputs("<B>I suspect that radiusd is no longer running.</b><br>\n",r);    	
	} else {
		fclose(Run);
		ap_rputs("<B>Radiusd appears to be running.</b><br>\n",r);    	
	}
	fclose(pidfile);
}

/* End of the server status code. */

/*    ap_rputs(ap_psignature("",r), r);    Don't need a signature. */
    ap_rputs("Mod_Radwho Version 2.3<br>\n", r);
    ap_rputs("</body></html>\n", r);
    /* Done, turn off timeout, close file and return */
    ap_kill_timeout(r);
    return 0;
}

/* Internal Authorization Check Routines.... */

/* First, check the userid.  If we have a user by that name, return OK, else deny */
static int db_basic_auth(request_rec *r)
{
char *user = r->connection->user;
int  res;
GDBM_FILE dbf;
datum key;

user = r->connection->user;
if ( user == NULL) return AUTH_REQUIRED;

/* open the datadabse to check the username. */

dbf = gdbm_open(AUTHFILE,512,GDBM_READER,622,0);

/* authorize if we have no file to block access.              */
/* You might want to change this to defaul to declined later. */

if ( dbf == NULL ) return OK;

/* fill in the gdbm datum record. */

key.dptr = user;
key.dsize= strlen(user);

res = gdbm_exists( dbf, key); /* check for the existence of the username */

gdbm_close(dbf);  /* MUST close the file--that's all I needed*/

	if ( !res ) {
		syslog(LOG_INFO,"Invalid username given: %s",user); 
		return DECLINED;
	}
/* username exists--return OK */

	syslog(LOG_DEBUG,"Username %s confirmed.",user);
	return OK;
}


/* Actually check authorization now. */
static int db_check_auth(request_rec *r)
{
char *user = r->connection->user;
int  res;
const char *sent_pw;
char *real_pw;
GDBM_FILE dbf;
datum key,username;


if ((res = ap_get_basic_auth_pw(r,&sent_pw)))
	return res;   /* I don't know what this does yet. */

/* on deny, return DECLINED    */
/* return AUTH_REQUIRED to ask */
/* return OK to permit access  */

user = r->connection->user;

if (user == NULL ) {
	syslog(LOG_DEBUG,"No username sent %s",sent_pw);
        return AUTH_REQUIRED;
}
/* open the datadabse and get my password. */

dbf = gdbm_open(AUTHFILE,512,GDBM_READER,622,0);

/* authorize if we have no file to block access. */
if ( dbf == NULL ) return OK;

username.dptr = user;
username.dsize= strlen(user);

key = gdbm_fetch( dbf, username);

if (key.dptr == NULL ) {     /* No such username */
	ap_log_rerror(APLOG_MARK, APLOG_NOERRNO|APLOG_ERR, r,
		"Radwho user %s:  UNKNOWN \"%s\": %s",
		user,r->uri,sent_pw);
	ap_note_basic_auth_failure(r);
	gdbm_close(dbf);  /* MUST close the file */
	syslog(LOG_DEBUG,"unknown user %s",user);
	return DECLINED;
}
key.dptr[key.dsize]=0;

gdbm_close(dbf);  /* MUST close the file */

real_pw = (char *)key.dptr;

if (strlen(sent_pw) < key.dsize ) {
	res = strncmp(sent_pw,real_pw,key.dsize);
} else {
	syslog(LOG_DEBUG,"%s sent short password [%s]",user,sent_pw);
	res = strncmp(sent_pw,real_pw,key.dsize);
}
free( key.dptr);  /* have to maually free this... */

if ( res != 0 )
	{
	syslog(LOG_DEBUG,"%s sent bad password (%s)[%s].",user,sent_pw,real_pw);
	ap_log_rerror(APLOG_MARK, APLOG_NOERRNO|APLOG_ERR, r,
		"Radwho user %s: authentication failure for \"%s\": %s %s",
		user,r->uri,real_pw,sent_pw);
	ap_note_basic_auth_failure(r);
	return AUTH_REQUIRED;
        } /* end of invalid password */
 
return OK;  /* permit access. */
}
/* End of the authorization check. */


static const handler_rec radwho_handlers[] =
{
    {"radwho", display_online},
    {NULL}
};

module icradius_who_module =
{
    STANDARD_MODULE_STUFF,
    NULL,                       /* initializer */
    NULL,                       /* dir config creater */
    NULL,                       /* dir merger --- default is to override */
    NULL,                       /* server config */
    NULL,                       /* merge server config */
    NULL,                       /* command table */
    radwho_handlers,            /* handlers */
    NULL,                       /* filename translation */
    db_basic_auth,              /* check_user_id */
    db_check_auth,              /* check auth */
    db_check_auth,              /* check access */
    NULL,                       /* type_checker */
    NULL,                       /* fixups */
    NULL,                       /* logger */
    NULL,                       /* header parser */
    NULL,                       /* child_init */
    NULL,                       /* child_exit */
    NULL                        /* post read-request */
};

